from urllib.parse import urlparse

from .base import BaseSpider
from ..items.category_item import CategoryItem


class CategorySpider(BaseSpider):
    instance = "category"
    current_page = 0

    def __init__(self, url, website_id):
        super().__init__(url)
        self.website_id = website_id

    @staticmethod
    def get_new_item_instance():
        return CategoryItem()

    def set_default_value(self):
        items = super().set_default_value()
        items["name"] = ""  # update when error
        items["css_selector"] = ""
        items["products_num"] = []
        items["from_page"] = 0  # update when error
        items["to_page"] = 0  # update when error
        items["next_page"] = ""  # update when error
        items["id_category"] = ""
        items["breadcrumb"] = ""
        items["total_products"] = 0  # update when error
        # use in pipeline to create product and send message
        items["product_urls"] = []
        return items



import os

from dotenv import load_dotenv

from ....spiders.base import BaseSpider
from ..website_item import WebsiteItem


class WebsiteTheamallSpider(BaseSpider):

    name = "theamall.com"

    def __init__(self, url):
        super().__init__(url=url)
        load_dotenv(os.path.abspath('.env-base'))

    def is_element_valid(self, element):
        if "/goods/category?cate=" in element:
            return True
        else:
            return False

    def process_response(self, response, items, response_elements):
        items = super().process_response(response, items, response_elements)
        items["name"] = "TheAMall"
        items["brand"] = "TheAMall"

        return items

    @staticmethod
    def get_new_item_instance():
        return WebsiteItem()

    def set_default_value(self):
        items = super().set_default_value()
        # all field use to update record after crawling
        items["brand"] = ""  # update when error
        items["categories_num"] = []
        items["total_categories"] = 0  # update when error

        # use in pipeline to create categories and send message
        items["category_urls"] = []
        return items

    def get_css_selector(self):
        return "#header > div.gnb > ul > li > div.over-area > div.contents > dl.cate-list a::attr(href)"


import logging
import os

from graypy import GELFTLSHandler


def get_default_log(instance_name):
    logger = logging.getLogger(f"crawling_{instance_name}_log")
    logger.setLevel(logging.INFO)

    handler = GELFTLSHandler(os.getenv("GRAYLOG_ADDR"), 12201)
    logger.addHandler(handler)
    logger.addHandler(logging.StreamHandler())
    return logger



import scrapy

from .base import BaseItem


class ProductItem(BaseItem):
    product_id = scrapy.Field()
    competitor = scrapy.Field()
    good_nm = scrapy.Field()
    normal_price = scrapy.Field()
    sale_price = scrapy.Field()
    shipping_price = scrapy.Field()
    shipping_type = scrapy.Field()
    shipping_condition = scrapy.Field()
    options = scrapy.Field()
    total_options = scrapy.Field()
    currency = scrapy.Field()



from ..connectors.message_brokers.base import BaseMessageBrokerConnector


class MessageConnectorMixin:

    def __init__(self):
        super().__init__()
        self.connector_params = {}
        self.connector_client = None
        self.setup_connector()

    def load_connector_env(self):
        pass

    def setup_connector(self, reinit=False):
        self.load_connector_env()
        if not self.connector_client or reinit:
            self.init_message_connector()

    def init_message_connector(self):
        self.connector_client = BaseMessageBrokerConnector()


    def send_message(self, message="", target=None):
        self.connector_client.send(message, target)

    def receive_message(self, receive_from, received_callback):
        self.connector_client.receive(receive_from, received_callback)